=== modified file 'lib/canonical/launchpad/browser/person.py'
--- lib/canonical/launchpad/browser/person.py	2009-03-03 04:48:14 +0000
+++ lib/canonical/launchpad/browser/person.py	2009-03-03 22:06:03 +0000
@@ -2790,8 +2790,10 @@
     def initialize(self):
         super(PersonIndexView, self).initialize()
         # This view requires the gmap2 Javascript in order to render the map
-        # with the person's usual location.
-        self.request.needs_gmap2 = True
+        # with the person's usual location. The location is only availble if
+        # the location is set and visible.
+        if self.has_visible_location:
+            self.request.needs_gmap2 = True
         if self.request.method == "POST":
             self.processForm()
 
@@ -2835,12 +2837,14 @@
                   "mailing list."))
         self.request.response.redirect(canonical_url(self.context))
 
+    @property
     def map_portlet_html(self):
         """Generate the HTML which shows the map portlet."""
+        assert self.has_visible_location, (
+            "Can't generate the map for a person who hasn't set a "
+            "visible location.")
         assert self.request.needs_gmap2, (
             "To use this method a view must flag that it needs gmap2.")
-        assert self.context.latitude is not None, (
-            "Can't generate the map for a person who hasn't set a location.")
 
         replacements = {'center_lat': self.context.latitude,
                         'center_lng': self.context.longitude}
@@ -2855,6 +2859,15 @@
                 });
             </script>""" % replacements
 
+    @cachedproperty
+    def has_visible_location(self):
+        """Does the person have latitude and a visible location."""
+        if self.context.is_team:
+            return len(self.context.mapped_participants) > 0
+        else:
+            return (check_permission('launchpad.View', self.context.location)
+                and self.context.latitude is not None)
+
     @property
     def should_show_map_portlet(self):
         """Should the map portlet be displayed?

=== modified file 'lib/canonical/launchpad/browser/team.py'
--- lib/canonical/launchpad/browser/team.py	2009-02-25 23:14:25 +0000
+++ lib/canonical/launchpad/browser/team.py	2009-03-02 19:02:21 +0000
@@ -911,7 +911,8 @@
     def initialize(self):
         # Tell our main-template to include Google's gmap2 javascript so that
         # we can render the map.
-        self.request.needs_gmap2 = True
+        if len(self.mapped_participants) > 0:
+            self.request.needs_gmap2 = True
 
     @cachedproperty
     def mapped_participants(self):

=== modified file 'lib/canonical/launchpad/doc/person-pages.txt'
--- lib/canonical/launchpad/doc/person-pages.txt	2009-01-12 02:53:41 +0000
+++ lib/canonical/launchpad/doc/person-pages.txt	2009-03-02 19:02:21 +0000
@@ -136,6 +136,61 @@
     []
 
 
+== Location ==
+
+The Person profile page contains the location portlet that shows a map.
+The map requires the google GMap JavaScript to display, so the views set
+the state of the request's needs_gmap2 attribute to True only when the
+user has set his latitude and it is visible The map is not rendered if the
+user has not set his location.
+
+    >>> sample_person.latitude is None
+    True
+
+    >>> person_view = create_initialized_view(sample_person, '+index')
+    >>> person_view.request.needs_gmap2
+    False
+
+The map_portlet_html property that creates the map cannot be called.
+
+    >>> print person_view.map_portlet_html
+    Traceback (most recent call last):
+     ...
+    AssertionError: Can't generate the map for a person who hasn't set
+                    a visible location.
+
+If the user set's his location, but does not make it visible, needs_gmap2
+will still be False and the map_portlet_html property cannot be called.
+
+    >>> login_person(sample_person)
+    >>> sample_person.setLocation(
+    ...     38.81, 77.1, 'America/New_York', sample_person)
+    >>> sample_person.setLocationVisibility(False)
+    >>> login('no-priv@canonical.com')
+
+    >>> person_view = create_initialized_view(sample_person, '+index')
+    >>> person_view.request.needs_gmap2
+    False
+    >>> print person_view.map_portlet_html
+    Traceback (most recent call last):
+     ...
+    AssertionError: Can't generate the map for a person who hasn't set
+                    a visible location.
+
+When the user set's his visibility to True, needs_gmap2 will be true and
+the map_portlet_html can be called.
+
+    >>> login_person(sample_person)
+    >>> sample_person.setLocationVisibility(True)
+
+    >>> person_view = create_initialized_view(sample_person, '+index')
+    >>> person_view.request.needs_gmap2
+    True
+    >>> print person_view.map_portlet_html
+    <script type="text/javascript">
+      YUI().use('node', 'lp.mapping', function(Y) { ...
+
+
 == Things a person is working on ==
 
 PersonView is the base for many views for Person objects. It provides

=== modified file 'lib/canonical/launchpad/doc/team-pages.txt'
--- lib/canonical/launchpad/doc/team-pages.txt	2009-01-08 03:23:37 +0000
+++ lib/canonical/launchpad/doc/team-pages.txt	2009-03-02 19:02:21 +0000
@@ -22,7 +22,6 @@
     >>> for member in team_home.recently_approved_members:
     ...     membership = membershipset.getByPersonAndTeam(member, ubuntu_team)
     ...     print "%s: %s" % (member.name, membership.status.title)
-    ...            
     name20: Approved
     spiv: Approved
     limi: Approved
@@ -72,33 +71,72 @@
     form fields.
 
 
+== +map-portlet ==
+
+The team profile page contain the location portlet that shows a map. The
+map requires the google GMap JavaScript to display, so the views set the
+state of the request's needs_gmap2 attribute to true if there are
+members who have set their location.
+
+    >>> team_view = create_initialized_view(ubuntu_team, '+index')
+    >>> team_view.has_visible_location
+    False
+    >>> team_view.request.needs_gmap2
+    False
+
+After a member has set his location, the map will be rendered.
+
+    >>> login_person(sample_person)
+    >>> sample_person.setLocation(
+    ...     38.81, 77.1, 'America/New_York', sample_person)
+
+    >>> team_view = create_initialized_view(ubuntu_team, '+index')
+    >>> team_view.has_visible_location
+    True
+    >>> team_view.request.needs_gmap2
+    True
+
+
 == +map ==
 
 A team's +map page will show a map with markers on the location of all
 members of that team.  That page also lists the local times of members.
 
+    >>> login('foo.bar@canonical.com')
     >>> context = factory.makeTeam(salgado)
     >>> len(context.mapped_participants)
     0
-    >>> view = create_view(context, '+map')
+    >>> view = create_initialized_view(context, '+map')
     >>> view.times
     []
 
+There are no mapped member yet, so needs_gmap2 is False, so the map will
+not be rendered.
+
+    >>> view.request.needs_gmap2
+    False
+
+Once a member is mapped, needs_gmap2 is True and the map will be rendered.
+The number of times is incremented for each timezone added.
+
     >>> london_member = factory.makePerson(
     ...     latitude=51.49, longitude=-0.13, time_zone='Europe/London')
     >>> context.addMember(london_member, sabdfl)
     >>> len(context.mapped_participants)
     1
-    >>> view = create_view(context, '+map')
+    >>> view = create_initialized_view(context, '+map')
     >>> len(view.times)
     1
 
+    >>> view.request.needs_gmap2
+    True
+
     >>> brazil_member = factory.makePerson(
     ...     latitude=-23.60, longitude=-46.64, time_zone='America/Sao_Paulo')
     >>> context.addMember(brazil_member, sabdfl)
     >>> len(context.mapped_participants)
     2
-    >>> view = create_view(context, '+map')
+    >>> view = create_initialized_view(context, '+map')
     >>> len(view.times)
     2
 

=== modified file 'lib/canonical/launchpad/javascript/lp/mapping.js'
--- lib/canonical/launchpad/javascript/lp/mapping.js	2009-03-02 02:13:27 +0000
+++ lib/canonical/launchpad/javascript/lp/mapping.js	2009-03-03 16:49:19 +0000
@@ -1,9 +1,23 @@
 YUI.add('lp.mapping', function(Y) {
     var mapping = Y.namespace('lp.mapping');
 
-    // jslint does not like functions that look like classes.
-    var gBrowserIsCompatible = GBrowserIsCompatible;
-    var gDownloadUrl = GDownloadUrl;
+    mapping.RETURN_FALSE = function() {return false;};
+    mapping.RETURN_NULL = function() {return null;};
+
+    // Replace the crucial GMap functions so that the supporting functions
+    // will work if the GMap script is not loaded.
+    var gBrowserIsCompatible = mapping.RETURN_FALSE;
+    var gDownloadUrl = RETURN_NULL;
+
+    mapping.has_gmaps = (typeof(GBrowserIsCompatible) == 'function');
+
+    if (mapping.has_gmaps) {
+        // The GMap2 is is loaded; use the real functions.
+        // jslint does not like functions that look like classes.
+        gBrowserIsCompatible = GBrowserIsCompatible;
+        gDownloadUrl = GDownloadUrl;
+    }
+
 
     /**
      * Add a marker for each participant.

=== modified file 'lib/canonical/launchpad/templates/main-template.pt'
--- lib/canonical/launchpad/templates/main-template.pt	2009-02-28 16:04:29 +0000
+++ lib/canonical/launchpad/templates/main-template.pt	2009-03-02 19:02:21 +0000
@@ -155,8 +155,7 @@
       <script type="text/javascript"
         tal:attributes="src string:${icingroot_contrib}/JSONScriptRequest.js"></script>
     </tal:needs_json>
-    <tal:needs-gmap2 condition="request/needs_gmap2"
-      define="lp_js string:${icingroot}/build">
+    <tal:needs-gmap2 condition="request/needs_gmap2">
       <tal:devmode condition="devmode">
         <script type="text/javascript"
                 tal:attributes="src string:http://maps.google.com/maps?file=api&amp;v=2&amp;key=${modules/canonical.config/config/google/maps_api_key};"
@@ -167,9 +166,10 @@
                 tal:attributes="src string:https://maps-api-ssl.google.com/maps?oe=utf-8&amp;client=gme-canonical&amp;file=api&amp;v=2&amp;key=${modules/canonical.config/config/google/maps_api_key};"
                 ></script>
       </tal:not-devmode>
-      <script type="text/javascript"
-              tal:attributes="src string:${lp_js}/lp/mapping.js"></script>
     </tal:needs-gmap2>
+    <script type="text/javascript"
+            tal:define="lp_js string:${icingroot}/build"
+            tal:attributes="src string:${lp_js}/lp/mapping.js"></script>
     <link rel="shortcut icon" href="/@@/launchpad.png" />
     <metal:block define-slot="head_epilogue"></metal:block>
   </head>

